@@ -20,6 +20,7 @@
 #ifndef _GCAL_APPLICATION_H_
 #define _GCAL_APPLICATION_H_
 
+#include "gcal-weather-service.h"
 #include "gcal-manager.h"
 
 #include <gtk/gtk.h>
@@ -34,6 +35,8 @@ GcalApplication*     gcal_application_new                        (void);
 
 GcalManager*         gcal_application_get_manager                (GcalApplication    *self);
 
+GcalWeatherService*  gcal_application_get_weather_service        (GcalApplication    *self);
+
 void                 gcal_application_set_uuid                   (GcalApplication    *self,
                                                                   const gchar        *app_uuid);
 
