@@ -1,6 +1,5 @@
-/* -*- mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/*
- * gcal-edit-dialog.h
+/* gcal-edit-dialog.h
+ *
  * Copyright (C) 2015 Erick Pérez Castellanos <erickpc@gnome.org>
  *
  * gnome-calendar is free software: you can redistribute it and/or modify it
@@ -20,6 +19,7 @@
 #ifndef __GCAL_EDIT_DIALOG_H__
 #define __GCAL_EDIT_DIALOG_H__
 
+#include "gcal-enums.h"
 #include "gcal-event.h"
 #include "gcal-manager.h"
 
@@ -49,7 +49,7 @@ void                 gcal_edit_dialog_set_manager             (GcalEditDialog *d
                                                                GcalManager    *manager);
 
 void                 gcal_edit_dialog_set_time_format         (GcalEditDialog *dialog,
-                                                               gboolean        use_24h_format);
+                                                               GcalTimeFormat  time_format);
 
 GDateTime*           gcal_edit_dialog_get_date_end            (GcalEditDialog *dialog);
 
@@ -57,6 +57,8 @@ GDateTime*           gcal_edit_dialog_get_date_start          (GcalEditDialog *d
 
 gboolean             gcal_edit_dialog_get_recurrence_changed   (GcalEditDialog *self);
 
+GcalRecurrenceModType gcal_edit_dialog_get_recurrence_mod_type (GcalEditDialog *self);
+
 G_END_DECLS
 
 #endif /* __GCAL_EDIT_DIALOG_H__ */
