@@ -16,10 +16,13 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GcalShellSearchProvider"
+
 #include "gcal-shell-search-provider.h"
 #include "gcal-shell-search-provider-generated.h"
 
 #include "gcal-application.h"
+#include "gcal-debug.h"
 #include "gcal-event.h"
 #include "gcal-window.h"
 #include "gcal-utils.h"
@@ -31,30 +34,37 @@ typedef struct
   icaltimetype              date;
 } PendingSearch;
 
-typedef struct
-{
-  GcalShellSearchProvider2 *skel;
-  GcalManager              *manager;
-
-  PendingSearch            *pending_search;
-  guint                     scheduled_search_id;
-  GHashTable               *events;
-} GcalShellSearchProviderPrivate;
-
 struct _GcalShellSearchProvider
 {
-  GObject parent;
+  GObject             parent;
 
-  /*< private >*/
-  GcalShellSearchProviderPrivate *priv;
+  GcalShellSearchProvider2 *skel;
+  GcalManager        *manager;
+
+  PendingSearch      *pending_search;
+  guint               scheduled_search_id;
+  GHashTable         *events;
 };
 
 static void   gcal_subscriber_interface_init (ECalDataModelSubscriberInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GcalShellSearchProvider, gcal_shell_search_provider, G_TYPE_OBJECT,
-                         G_ADD_PRIVATE (GcalShellSearchProvider)
                          G_IMPLEMENT_INTERFACE (E_TYPE_CAL_DATA_MODEL_SUBSCRIBER, gcal_subscriber_interface_init));
 
+enum
+{
+  PROP_0,
+  PROP_MANAGER,
+  N_PROPS
+};
+
+static GParamSpec* properties[N_PROPS] = { NULL, };
+
+
+/*
+ * Auxiliary methods
+ */
+
 static gint
 sort_event_data (GcalEvent *a,
                  GcalEvent *b,
@@ -64,159 +74,160 @@ sort_event_data (GcalEvent *a,
 }
 
 static gboolean
-execute_search (GcalShellSearchProvider *search_provider)
+execute_search (GcalShellSearchProvider *self)
 {
-  GcalShellSearchProviderPrivate *priv;
-
-  guint i;
-  gchar *search_query;
-
   icaltimezone *zone;
+  g_autofree gchar *search_query = NULL;
   time_t range_start, range_end;
+  guint i;
 
-  priv = search_provider->priv;
+  GCAL_ENTRY;
 
-  if (!gcal_manager_load_completed (priv->manager))
-    return TRUE;
+  if (gcal_manager_get_loading (self->manager))
+    GCAL_RETURN (TRUE);
 
-  zone = gcal_manager_get_system_timezone (priv->manager);
-  priv->pending_search->date = icaltime_current_time_with_zone (zone);
-  icaltime_adjust (&(priv->pending_search->date), -7, 0, 0, 0); /* -1 weeks from today */
-  range_start = icaltime_as_timet_with_zone (priv->pending_search->date, zone);
+  zone = gcal_manager_get_system_timezone (self->manager);
+  self->pending_search->date = icaltime_current_time_with_zone (zone);
+  icaltime_adjust (&(self->pending_search->date), -7, 0, 0, 0); /* -1 weeks from today */
+  range_start = icaltime_as_timet_with_zone (self->pending_search->date, zone);
 
-  icaltime_adjust (&(priv->pending_search->date), 21 * 2, 0, 0, 0); /* +3 weeks from today */
-  range_end = icaltime_as_timet_with_zone (priv->pending_search->date, zone);
+  icaltime_adjust (&(self->pending_search->date), 21 * 2, 0, 0, 0); /* +3 weeks from today */
+  range_end = icaltime_as_timet_with_zone (self->pending_search->date, zone);
 
-  gcal_manager_set_shell_search_subscriber (priv->manager, E_CAL_DATA_MODEL_SUBSCRIBER (search_provider),
+  gcal_manager_set_shell_search_subscriber (self->manager, E_CAL_DATA_MODEL_SUBSCRIBER (self),
                                             range_start, range_end);
 
   search_query = g_strdup_printf ("(or (contains? \"summary\" \"%s\") (contains? \"description\" \"%s\"))",
-                                  priv->pending_search->terms[0], priv->pending_search->terms[0]);
-  for (i = 1; i < g_strv_length (priv->pending_search->terms); i++)
+                                  self->pending_search->terms[0], self->pending_search->terms[0]);
+  for (i = 1; i < g_strv_length (self->pending_search->terms); i++)
     {
-      gchar *complete_query;
-      gchar *second_query = g_strdup_printf ("(or (contains? \"summary\" \"%s\") (contains? \"description\" \"%s\"))",
-                                             priv->pending_search->terms[0], priv->pending_search->terms[0]);
+      g_autofree gchar *complete_query = NULL;
+      g_autofree gchar *second_query = NULL;
+
+      second_query = g_strdup_printf ("(or (contains? \"summary\" \"%s\") (contains? \"description\" \"%s\"))",
+                                      self->pending_search->terms[0], self->pending_search->terms[0]);
       complete_query = g_strdup_printf ("(and %s %s)", search_query, second_query);
 
-      g_free (second_query);
-      g_free (search_query);
-
-      search_query = complete_query;
+      g_clear_pointer (&search_query, g_free);
+      search_query = g_steal_pointer (&complete_query);
     }
 
-  gcal_manager_set_shell_search_query (priv->manager,  search_query);
-  g_free (search_query);
+  gcal_manager_set_shell_search_query (self->manager,  search_query);
 
-  priv->scheduled_search_id = 0;
+  self->scheduled_search_id = 0;
   g_application_hold (g_application_get_default ());
-  return FALSE;
+
+  GCAL_RETURN (FALSE);
 }
 
 static void
-schedule_search (GcalShellSearchProvider *search_provider,
+schedule_search (GcalShellSearchProvider *self,
                  GDBusMethodInvocation   *invocation,
                  gchar                  **terms)
 {
-  GcalShellSearchProviderPrivate *priv = search_provider->priv;
+  GCAL_ENTRY;
 
   /* don't attempt searches for a single character */
   if (g_strv_length (terms) == 1 && g_utf8_strlen (terms[0], -1) == 1)
     {
       g_dbus_method_invocation_return_value (invocation, g_variant_new ("(as)", NULL));
-      return;
+      GCAL_RETURN ();
     }
 
-  if (priv->pending_search != NULL)
+  if (self->pending_search != NULL)
     {
-      g_object_unref (priv->pending_search->invocation);
-      g_strfreev (priv->pending_search->terms);
+      g_object_unref (self->pending_search->invocation);
+      g_strfreev (self->pending_search->terms);
 
-      if (priv->scheduled_search_id == 0)
+      if (self->scheduled_search_id == 0)
         g_application_release (g_application_get_default ());
     }
   else
     {
-      priv->pending_search = g_new0 (PendingSearch, 1);
+      self->pending_search = g_new0 (PendingSearch, 1);
     }
 
-  if (priv->scheduled_search_id != 0)
+  if (self->scheduled_search_id != 0)
     {
-      g_source_remove (priv->scheduled_search_id);
-      priv->scheduled_search_id = 0;
+      g_source_remove (self->scheduled_search_id);
+      self->scheduled_search_id = 0;
     }
 
-  priv->pending_search->invocation = g_object_ref (invocation);
-  priv->pending_search->terms = g_strdupv (terms);
+  self->pending_search->invocation = g_object_ref (invocation);
+  self->pending_search->terms = g_strdupv (terms);
 
-  if (!gcal_manager_load_completed (priv->manager))
+  if (gcal_manager_get_loading (self->manager))
     {
-      priv->scheduled_search_id = g_timeout_add_seconds (1, (GSourceFunc) execute_search, search_provider);
-      return;
+      self->scheduled_search_id = g_timeout_add_seconds (1, (GSourceFunc) execute_search, self);
+      GCAL_RETURN ();
     }
 
-  execute_search (search_provider);
-  return;
+  execute_search (self);
+
+  GCAL_EXIT;
 }
 
+
+/*
+ * Callbacks
+ */
+
 static gboolean
-get_initial_result_set_cb (GcalShellSearchProvider  *search_provider,
+get_initial_result_set_cb (GcalShellSearchProvider  *self,
                            GDBusMethodInvocation    *invocation,
                            gchar                   **terms,
                            GcalShellSearchProvider2 *skel)
 {
-  schedule_search (search_provider, invocation, terms);
+  schedule_search (self, invocation, terms);
   return TRUE;
 }
 
 static gboolean
-get_subsearch_result_set_cb (GcalShellSearchProvider  *search_provider,
+get_subsearch_result_set_cb (GcalShellSearchProvider  *self,
                              GDBusMethodInvocation    *invocation,
                              gchar                   **previous_results,
                              gchar                   **terms,
                              GcalShellSearchProvider2 *skel)
 {
-  schedule_search (search_provider, invocation, terms);
+  schedule_search (self, invocation, terms);
   return TRUE;
 }
 
 static gboolean
-get_result_metas_cb (GcalShellSearchProvider  *search_provider,
+get_result_metas_cb (GcalShellSearchProvider  *self,
                      GDBusMethodInvocation    *invocation,
                      gchar                   **results,
                      GcalShellSearchProvider2 *skel)
 {
-  GcalShellSearchProviderPrivate *priv;
   GDateTime *local_datetime;
   GVariantBuilder abuilder, builder;
-  GVariant *icon_variant;
   GcalEvent *event;
-  GdkPixbuf *gicon;
   gchar *uuid, *desc;
   gchar *start_date;
   gint i;
 
-  priv = search_provider->priv;
+  GCAL_ENTRY;
 
   g_variant_builder_init (&abuilder, G_VARIANT_TYPE ("aa{sv}"));
   for (i = 0; i < g_strv_length (results); i++)
     {
+      g_autoptr (GVariant) icon_variant = NULL;
+      g_autoptr (GdkPixbuf) gicon = NULL;
+      cairo_surface_t *surface;
+
       uuid = results[i];
-      event = g_hash_table_lookup (priv->events, uuid);
+      event = g_hash_table_lookup (self->events, uuid);
 
       g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{sv}"));
       g_variant_builder_add (&builder, "{sv}", "id", g_variant_new_string (uuid));
       g_variant_builder_add (&builder, "{sv}", "name", g_variant_new_string (gcal_event_get_summary (event)));
 
-      gicon = get_circle_pixbuf_from_color (gcal_event_get_color (event), 128);
+      surface = get_circle_surface_from_color (gcal_event_get_color (event), 96);
+      gicon = gdk_pixbuf_get_from_surface (surface, 0, 0, 96, 96);
       icon_variant = g_icon_serialize (G_ICON (gicon));
       g_variant_builder_add (&builder, "{sv}", "icon", icon_variant);
-      g_object_unref (gicon);
-      g_variant_unref (icon_variant);
 
       local_datetime = g_date_time_to_local (gcal_event_get_date_start (event));
-
-      /* FIXME: respect 24h time format */
       start_date = g_date_time_format (local_datetime, gcal_event_get_all_day (event) ? "%x" : "%c");
 
       if (gcal_event_get_location (event))
@@ -226,29 +237,31 @@ get_result_metas_cb (GcalShellSearchProvider  *search_provider,
 
       g_variant_builder_add (&builder, "{sv}", "description", g_variant_new_string (desc));
       g_variant_builder_add_value (&abuilder, g_variant_builder_end (&builder));
+
+      g_clear_pointer (&surface, cairo_surface_destroy);
     }
   g_dbus_method_invocation_return_value (invocation, g_variant_new ("(aa{sv})", &abuilder));
 
-  return TRUE;
+  GCAL_RETURN (TRUE);
 }
 
 static gboolean
-activate_result_cb (GcalShellSearchProvider  *search_provider,
+activate_result_cb (GcalShellSearchProvider  *self,
                     GDBusMethodInvocation    *invocation,
                     gchar                    *result,
                     gchar                   **terms,
                     guint32                   timestamp,
                     GcalShellSearchProvider2 *skel)
 {
-  GcalShellSearchProviderPrivate *priv;
+  g_autoptr (GcalEvent) event = NULL;
   GApplication *application;
-  GcalEvent *event;
   GDateTime *dtstart;
 
-  priv = search_provider->priv;
+  GCAL_ENTRY;
+
   application = g_application_get_default ();
 
-  event = gcal_manager_get_event_from_shell_search (priv->manager, result);
+  event = gcal_manager_get_event_from_shell_search (self->manager, result);
   dtstart = gcal_event_get_date_start (event);
 
   gcal_application_set_uuid (GCAL_APPLICATION (application), result);
@@ -256,55 +269,53 @@ activate_result_cb (GcalShellSearchProvider  *search_provider,
 
   g_application_activate (application);
 
-  g_clear_object (&event);
-
-  return TRUE;
+  GCAL_RETURN (TRUE);
 }
 
 static gboolean
-launch_search_cb (GcalShellSearchProvider  *search_provider,
+launch_search_cb (GcalShellSearchProvider  *self,
                   GDBusMethodInvocation    *invocation,
                   gchar                   **terms,
                   guint32                   timestamp,
                   GcalShellSearchProvider2 *skel)
 {
+  g_autofree gchar *terms_joined = NULL;
   GApplication *application;
-  gchar *terms_joined;
-  GList *windows;
+  GcalWindow *window;
+
+  GCAL_ENTRY;
 
   application = g_application_get_default ();
   g_application_activate (application);
 
   terms_joined = g_strjoinv (" ", terms);
-  windows = g_list_reverse (gtk_application_get_windows (GTK_APPLICATION (application)));
-  if (windows != NULL)
+  window = (GcalWindow *) gtk_application_get_active_window (GTK_APPLICATION (application));
+  if (window)
     {
-      gcal_window_set_search_mode (GCAL_WINDOW (windows->data), TRUE);
-      gcal_window_set_search_query (GCAL_WINDOW (windows->data), terms_joined);
-
-      g_list_free (windows);
+      gcal_window_set_search_mode (window, TRUE);
+      gcal_window_set_search_query (window, terms_joined);
     }
 
-  g_free (terms_joined);
-  return TRUE;
+  GCAL_RETURN (TRUE);
 }
 
 static gboolean
-query_completed_cb (GcalShellSearchProvider *search_provider,
+query_completed_cb (GcalShellSearchProvider *self,
                     GcalManager             *manager)
 {
-  GcalShellSearchProviderPrivate *priv = search_provider->priv;
   GList *events, *l;
   GVariantBuilder builder;
   time_t current_time_t;
 
-  g_hash_table_remove_all (priv->events);
+  GCAL_ENTRY;
 
-  events = gcal_manager_get_shell_search_events (priv->manager);
+  g_hash_table_remove_all (self->events);
+
+  events = gcal_manager_get_shell_search_events (self->manager);
   if (events == NULL)
     {
-      g_dbus_method_invocation_return_value (priv->pending_search->invocation, g_variant_new ("(as)", NULL));
-      goto out;
+      g_dbus_method_invocation_return_value (self->pending_search->invocation, g_variant_new ("(as)", NULL));
+      GCAL_GOTO (out);
     }
 
   g_variant_builder_init (&builder, G_VARIANT_TYPE ("as"));
@@ -317,25 +328,31 @@ query_completed_cb (GcalShellSearchProvider *search_provider,
 
       uid = gcal_event_get_uid (l->data);
 
-      if (g_hash_table_contains (priv->events, uid))
+      if (g_hash_table_contains (self->events, uid))
         continue;
 
       g_variant_builder_add (&builder, "s", uid);
 
-      g_hash_table_insert (priv->events, g_strdup (uid), l->data);
+      g_hash_table_insert (self->events, g_strdup (uid), l->data);
     }
   g_list_free (events);
 
-  g_dbus_method_invocation_return_value (priv->pending_search->invocation, g_variant_new ("(as)", &builder));
+  g_dbus_method_invocation_return_value (self->pending_search->invocation, g_variant_new ("(as)", &builder));
 
 out:
-  g_object_unref (priv->pending_search->invocation);
-  g_strfreev (priv->pending_search->terms);
-  g_clear_pointer (&(priv->pending_search), g_free);
+  g_clear_object (&self->pending_search->invocation);
+  g_clear_pointer (&self->pending_search->terms, g_strfreev);
+  g_clear_pointer (&self->pending_search, g_free);
   g_application_release (g_application_get_default ());
-  return FALSE;
+
+  GCAL_RETURN (FALSE);
 }
 
+
+/*
+ * ECalDataModelSubscriber iface
+ */
+
 static void
 gcal_shell_search_provider_component_changed (ECalDataModelSubscriber *subscriber,
                                               ECalClient              *client,
@@ -365,16 +382,6 @@ gcal_shell_search_provider_thaw (ECalDataModelSubscriber *subscriber)
   ;
 }
 
-static void
-gcal_shell_search_provider_finalize (GObject *object)
-{
-  GcalShellSearchProviderPrivate *priv = GCAL_SHELL_SEARCH_PROVIDER (object)->priv;
-
-  g_hash_table_destroy (priv->events);
-  g_clear_object (&priv->skel);
-  G_OBJECT_CLASS (gcal_shell_search_provider_parent_class)->finalize (object);
-}
-
 static void
 gcal_subscriber_interface_init (ECalDataModelSubscriberInterface *iface)
 {
@@ -385,65 +392,120 @@ gcal_subscriber_interface_init (ECalDataModelSubscriberInterface *iface)
   iface->thaw = gcal_shell_search_provider_thaw;
 }
 
+
+/*
+ * GObject overrides
+ */
+
+static void
+gcal_shell_search_provider_finalize (GObject *object)
+{
+  GcalShellSearchProvider *self = (GcalShellSearchProvider *) object;
+
+  g_clear_pointer (&self->events, g_hash_table_destroy);
+  g_clear_object (&self->manager);
+  g_clear_object (&self->skel);
+
+  G_OBJECT_CLASS (gcal_shell_search_provider_parent_class)->finalize (object);
+}
+
+static void
+gcal_shell_search_provider_get_property (GObject    *object,
+                                         guint       property_id,
+                                         GValue     *value,
+                                         GParamSpec *pspec)
+{
+  GcalShellSearchProvider *self = GCAL_SHELL_SEARCH_PROVIDER (object);
+
+  switch (property_id)
+    {
+    case PROP_MANAGER:
+      g_value_set_object (value, self->manager);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    }
+}
+
+static void
+gcal_shell_search_provider_set_property (GObject      *object,
+                                         guint         property_id,
+                                         const GValue *value,
+                                         GParamSpec   *pspec)
+{
+  GcalShellSearchProvider *self = GCAL_SHELL_SEARCH_PROVIDER (object);
+
+  switch (property_id)
+    {
+    case PROP_MANAGER:
+      if (g_set_object (&self->manager, g_value_get_object (value)))
+        {
+          gcal_manager_setup_shell_search (self->manager, E_CAL_DATA_MODEL_SUBSCRIBER (self));
+          g_signal_connect_swapped (self->manager, "query-completed", G_CALLBACK (query_completed_cb), self);
+
+          g_object_notify_by_pspec (object, properties[PROP_MANAGER]);
+        }
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    }
+}
+
 static void
 gcal_shell_search_provider_class_init (GcalShellSearchProviderClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->finalize = gcal_shell_search_provider_finalize;
+  object_class->get_property = gcal_shell_search_provider_get_property;
+  object_class->set_property = gcal_shell_search_provider_set_property;
+
+  properties[PROP_MANAGER] = g_param_spec_object ("manager",
+                                                  "The manager object",
+                                                  "The manager object",
+                                                  GCAL_TYPE_MANAGER,
+                                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
 static void
 gcal_shell_search_provider_init (GcalShellSearchProvider *self)
 {
-  GcalShellSearchProviderPrivate *priv = gcal_shell_search_provider_get_instance_private (self);
+  self->events = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+  self->skel = gcal_shell_search_provider2_skeleton_new ();
 
-  priv->events = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
-  priv->skel = gcal_shell_search_provider2_skeleton_new ();
-
-  g_signal_connect_swapped (priv->skel, "handle-get-initial-result-set", G_CALLBACK (get_initial_result_set_cb), self);
-  g_signal_connect_swapped (priv->skel, "handle-get-subsearch-result-set", G_CALLBACK (get_subsearch_result_set_cb), self);
-  g_signal_connect_swapped (priv->skel, "handle-get-result-metas", G_CALLBACK (get_result_metas_cb), self);
-  g_signal_connect_swapped (priv->skel, "handle-activate-result", G_CALLBACK (activate_result_cb), self);
-  g_signal_connect_swapped (priv->skel, "handle-launch-search", G_CALLBACK (launch_search_cb), self);
-
-  self->priv = priv;
+  g_signal_connect_swapped (self->skel, "handle-get-initial-result-set", G_CALLBACK (get_initial_result_set_cb), self);
+  g_signal_connect_swapped (self->skel, "handle-get-subsearch-result-set", G_CALLBACK (get_subsearch_result_set_cb), self);
+  g_signal_connect_swapped (self->skel, "handle-get-result-metas", G_CALLBACK (get_result_metas_cb), self);
+  g_signal_connect_swapped (self->skel, "handle-activate-result", G_CALLBACK (activate_result_cb), self);
+  g_signal_connect_swapped (self->skel, "handle-launch-search", G_CALLBACK (launch_search_cb), self);
 }
 
 GcalShellSearchProvider*
-gcal_shell_search_provider_new (void)
+gcal_shell_search_provider_new (GcalManager *manager)
 {
-  return g_object_new (GCAL_TYPE_SHELL_SEARCH_PROVIDER, NULL);
+  return g_object_new (GCAL_TYPE_SHELL_SEARCH_PROVIDER,
+                       "manager", manager,
+                       NULL);
 }
 
 gboolean
-gcal_shell_search_provider_dbus_export (GcalShellSearchProvider *search_provider,
-                                        GDBusConnection         *connection,
-                                        const gchar             *object_path,
-                                        GError                 **error)
+gcal_shell_search_provider_dbus_export (GcalShellSearchProvider  *self,
+                                        GDBusConnection          *connection,
+                                        const gchar              *object_path,
+                                        GError                  **error)
 {
-  GcalShellSearchProviderPrivate *priv = GCAL_SHELL_SEARCH_PROVIDER (search_provider)->priv;
-
-  return g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (priv->skel), connection, object_path, error);
+  return g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (self->skel), connection, object_path, error);
 }
 
 void
-gcal_shell_search_provider_dbus_unexport (GcalShellSearchProvider *search_provider,
+gcal_shell_search_provider_dbus_unexport (GcalShellSearchProvider *self,
                                           GDBusConnection         *connection,
                                           const gchar             *object_path)
 {
-  GcalShellSearchProviderPrivate *priv = GCAL_SHELL_SEARCH_PROVIDER (search_provider)->priv;
-
-  if (g_dbus_interface_skeleton_has_connection (G_DBUS_INTERFACE_SKELETON (priv->skel), connection))
-    g_dbus_interface_skeleton_unexport_from_connection (G_DBUS_INTERFACE_SKELETON (priv->skel), connection);
-}
-
-void
-gcal_shell_search_provider_connect (GcalShellSearchProvider *search_provider,
-                                    GcalManager             *manager)
-{
-  search_provider->priv->manager = manager;
-
-  gcal_manager_setup_shell_search (manager, E_CAL_DATA_MODEL_SUBSCRIBER (search_provider));
-  g_signal_connect_swapped (manager, "query-completed", G_CALLBACK (query_completed_cb), search_provider);
+  if (g_dbus_interface_skeleton_has_connection (G_DBUS_INTERFACE_SKELETON (self->skel), connection))
+    g_dbus_interface_skeleton_unexport_from_connection (G_DBUS_INTERFACE_SKELETON (self->skel), connection);
 }
