@@ -1,6 +1,4 @@
-/* -*- mode: c; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/*
- * gcal-week-view.c
+/* gcal-week-view.c
  *
  * Copyright (C) 2015 - Erick Pérez Castellanos
  *
@@ -21,12 +19,14 @@
 #define G_LOG_DOMAIN "GcalWeekView"
 
 #include "gcal-debug.h"
+#include "gcal-enums.h"
 #include "gcal-event-widget.h"
 #include "gcal-utils.h"
 #include "gcal-view.h"
 #include "gcal-week-header.h"
 #include "gcal-week-grid.h"
 #include "gcal-week-view.h"
+#include "gcal-weather-service.h"
 
 #include <glib/gi18n.h>
 
@@ -42,54 +42,41 @@ static const double dashed [] =
 
 struct _GcalWeekView
 {
-  GtkBox          parent;
+  GtkBox              parent;
 
-  GtkWidget      *header;
-  GtkWidget      *hours_bar;
-  GtkWidget      *scrolled_window;
-  GtkWidget      *week_grid;
+  GtkWidget          *header;
+  GtkWidget          *hours_bar;
+  GtkWidget          *scrolled_window;
+  GtkWidget          *week_grid;
 
-  /*
-   * first day of the week according to user locale, being
-   * 0 for Sunday, 1 for Monday and so on
-   */
-  gint            first_weekday;
-
-  /*
-   * clock format from GNOME desktop settings
-   */
-  gboolean        use_24h_format;
+  GcalTimeFormat      time_format;
 
   /* property */
-  icaltimetype   *date;
-  GcalManager    *manager; /* weak referenced */
+  icaltimetype       *date;
+  GcalManager        *manager;         /* owned */
+  GcalWeatherService *weather_service; /* owned */
 
-  guint           scroll_grid_timeout_id;
+  guint               scroll_grid_timeout_id;
 
-  gint            clicked_cell;
+  gint                clicked_cell;
 };
 
-static void           schedule_position_scroll                  (GcalWeekView      *self);
+static void          schedule_position_scroll                    (GcalWeekView       *self);
 
-static void           gcal_view_interface_init                  (GcalViewInterface *iface);
+static void          gcal_view_interface_init                    (GcalViewInterface  *iface);
 
-static void           gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface);
-
-enum
-{
-  EVENT_ACTIVATED,
-  LAST_SIGNAL
-};
+static void          gcal_data_model_subscriber_interface_init   (ECalDataModelSubscriberInterface *iface);
 
 enum
 {
   PROP_0,
   PROP_DATE,
   PROP_MANAGER,
+  PROP_TIME_FORMAT,
+  PROP_WEATHER_SERVICE,
   NUM_PROPS
 };
 
-static guint signals[LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE_WITH_CODE (GcalWeekView, gcal_week_view, GTK_TYPE_BOX,
                          G_IMPLEMENT_INTERFACE (GCAL_TYPE_VIEW, gcal_view_interface_init)
@@ -101,7 +88,7 @@ static void
 on_event_activated (GcalWeekView *self,
                     GtkWidget    *widget)
 {
-  g_signal_emit (self, signals[EVENT_ACTIVATED], 0, widget);
+  g_signal_emit_by_name (self, "event-activated", widget);
 }
 
 static void
@@ -317,8 +304,7 @@ gcal_week_view_component_added (ECalDataModelSubscriber *subscriber,
                                 ECalComponent           *comp)
 {
   GcalWeekView *self = GCAL_WEEK_VIEW (subscriber);
-
-  GcalEvent *event;
+  g_autoptr (GcalEvent) event = NULL;
 
   GCAL_ENTRY;
 
@@ -435,7 +421,7 @@ gcal_week_view_draw_hours (GcalWeekView *self,
     {
       gchar *hours;
 
-      if (self->use_24h_format)
+      if (self->time_format == GCAL_TIME_FORMAT_24H)
         {
           hours = g_strdup_printf ("%02d:00", i);
         }
@@ -443,7 +429,7 @@ gcal_week_view_draw_hours (GcalWeekView *self,
         {
           hours = g_strdup_printf ("%d %s",
                                    i % 12 == 0 ? 12 : i % 12,
-                                   i > 12 ? _("PM") : _("AM"));
+                                   i >= 12 ? _("PM") : _("AM"));
         }
 
       pango_layout_set_text (layout, hours, -1);
@@ -521,6 +507,7 @@ gcal_week_view_finalize (GObject       *object)
   g_clear_pointer (&self->date, g_free);
 
   g_clear_object (&self->manager);
+  g_clear_object (&self->weather_service);
 
   /* Chain up to parent's finalize() method. */
   G_OBJECT_CLASS (gcal_week_view_parent_class)->finalize (object);
@@ -545,10 +532,19 @@ gcal_week_view_set_property (GObject       *object,
 
       gcal_week_grid_set_manager (GCAL_WEEK_GRID (self->week_grid), self->manager);
       gcal_week_header_set_manager (GCAL_WEEK_HEADER (self->header), self->manager);
-
       g_object_notify (object, "manager");
       break;
 
+    case PROP_TIME_FORMAT:
+      self->time_format = g_value_get_enum (value);
+      gtk_widget_queue_draw (self->hours_bar);
+      break;
+
+    case PROP_WEATHER_SERVICE:
+      if (g_set_object (&self->weather_service, g_value_get_object (value)))
+        g_object_notify (object, "weather-service");
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -576,6 +572,14 @@ gcal_week_view_get_property (GObject       *object,
       g_value_set_object (value, self->manager);
       break;
 
+    case PROP_TIME_FORMAT:
+      g_value_set_enum (value, self->time_format);
+      break;
+
+    case PROP_WEATHER_SERVICE:
+      g_value_set_object (value, self->weather_service);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -588,20 +592,25 @@ gcal_week_view_class_init (GcalWeekViewClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+  g_type_ensure (GCAL_TYPE_WEEK_GRID);
+  g_type_ensure (GCAL_TYPE_WEEK_HEADER);
+
   object_class->finalize = gcal_week_view_finalize;
   object_class->set_property = gcal_week_view_set_property;
   object_class->get_property = gcal_week_view_get_property;
 
   g_object_class_override_property (object_class, PROP_DATE, "active-date");
   g_object_class_override_property (object_class, PROP_MANAGER, "manager");
+  g_object_class_override_property (object_class, PROP_WEATHER_SERVICE, "weather-service");
 
-  signals[EVENT_ACTIVATED] = g_signal_new ("event-activated",
-                                           GCAL_TYPE_WEEK_VIEW,
-                                           G_SIGNAL_RUN_FIRST,
-                                           0,  NULL, NULL, NULL,
-                                           G_TYPE_NONE,
-                                           1,
-                                           GCAL_TYPE_EVENT_WIDGET);
+  g_object_class_install_property (object_class,
+                                   PROP_TIME_FORMAT,
+                                   g_param_spec_enum ("time-format",
+                                                      "Time format",
+                                                      "Time format",
+                                                      GCAL_TYPE_TIME_FORMAT,
+                                                      GCAL_TIME_FORMAT_24H,
+                                                      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/week-view.ui");
 
@@ -622,54 +631,9 @@ gcal_week_view_init (GcalWeekView *self)
   gtk_widget_init_template (GTK_WIDGET (self));
 
   update_hours_sidebar_size (self);
+
+  self->weather_service = NULL;
+
+  g_object_bind_property (self, "weather-service", self->header, "weather-service", G_BINDING_DEFAULT);
 }
 
-/* Public API */
-/**
- * gcal_week_view_new:
- *
- * Create a week-view widget
- *
- * Returns: (transfer full):
- **/
-GtkWidget*
-gcal_week_view_new (void)
-{
-  return g_object_new (GCAL_TYPE_WEEK_VIEW, NULL);
-}
-
-/**
- * gcal_week_view_set_first_weekday:
- * @view:
- * @day_nr:
- *
- * Set the first day of the week according to the locale, being
- * 0 for Sunday, 1 for Monday and so on.
- **/
-void
-gcal_week_view_set_first_weekday (GcalWeekView *self,
-                                  gint          day_nr)
-{
-  g_return_if_fail (GCAL_IS_WEEK_VIEW (self));
-
-  self->first_weekday = day_nr;
-
-  gcal_week_header_set_first_weekday (GCAL_WEEK_HEADER (self->header), self->first_weekday);
-  gcal_week_grid_set_first_weekday (GCAL_WEEK_GRID (self->week_grid), self->first_weekday);
-}
-
-/**
- * gcal_week_view_set_use_24h_format:
- * @view:
- * @use_24h:
- *
- * Whether the view will show time using 24h or 12h format
- **/
-void
-gcal_week_view_set_use_24h_format (GcalWeekView *self,
-                                   gboolean      use_24h)
-{
-  g_return_if_fail (GCAL_IS_WEEK_VIEW (self));
-
-  self->use_24h_format = use_24h;
-}
